@@ -1,4 +1,5 @@
-/* xscreensaver, Copyright (c) 1992, 1995 Jamie Zawinski <jwz@netscape.com>
+/* xscreensaver, Copyright (c) 1992, 1995, 1997
+ *  Jamie Zawinski <jwz@netscape.com>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -21,47 +22,51 @@
    -  create a variable `char *progclass' which names this program's
       resource class.
 
-   -  create a variable `char defaults []' for the default resources.
+   -  create a variable `char defaults []' for the default resources, and
+      null-terminate it.
 
-   -  create a variable `XrmOptionDescRec options []' for the command-line,
-      and `int options_size' which is `XtNumber (options)'.
+   -  create a variable `XrmOptionDescRec options[]' for the command-line,
+      and null-terminate it.
 
    And that's it...
  */
 
-#include "version.h"
-
 #include <stdio.h>
 #include <X11/Intrinsic.h>
 #include <X11/IntrinsicP.h>
 #include <X11/CoreP.h>
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
-#include <X11/Xmu/Error.h>
+#ifdef HAVE_XMU
+# ifndef VMS
+#  include <X11/Xmu/Error.h>
+# else /* VMS */
+#  include <Xmu/Error.h>
+# endif
+#else
+# include "xmu.h"
+#endif
 #include "screenhack.h"
+#include "version.h"
+#include "vroot.h"
 
 char *progname;
 XrmDatabase db;
 Bool mono_p;
 
-#if __STDC__
-# define P(x) x
-#else
-# define P(x)()
-#endif
-
-
 static XrmOptionDescRec default_options [] = {
   { "-root",	".root",		XrmoptionNoArg, "True" },
   { "-window",	".root",		XrmoptionNoArg, "False" },
   { "-mono",	".mono",		XrmoptionNoArg, "True" },
   { "-install",	".installColormap",	XrmoptionNoArg, "True" },
-  { "-visual",	".visualID",		XrmoptionSepArg, 0 }
+  { "-noinstall",".installColormap",	XrmoptionNoArg, "False" },
+  { "-visual",	".visualID",		XrmoptionSepArg, 0 },
+  { 0, 0, 0, 0 }
 };
 
 static char *default_defaults[] = {
   "*root:		false",
-  "*geometry:		500x500", /* this should be .geometry, but nooooo... */
+  "*geometry:		600x480", /* this should be .geometry, but nooooo... */
   "*mono:		false",
   "*installColormap:	false",
   "*visualID:		default",
@@ -73,24 +78,36 @@ static int merged_options_size;
 static char **merged_defaults;
 
 static void
-merge_options P((void))
+merge_options (void)
 {
-  int options_sizeof = options_size * sizeof (options[0]);
-  int defaults_size;
-  merged_options_size = XtNumber (default_options) + options_size;
-  merged_options = (XrmOptionDescRec *)
-    malloc (sizeof (default_options) + options_sizeof);
-  memcpy (merged_options, options, options_sizeof);
-  memcpy (merged_options + options_size, default_options,
-	  sizeof (default_options));
+  int def_opts_size, opts_size;
+  int def_defaults_size, defaults_size;
 
-  for (defaults_size = 0; defaults [defaults_size]; defaults_size++);
+  for (def_opts_size = 0; default_options[def_opts_size].option;
+       def_opts_size++)
+    ;
+  for (opts_size = 0; options[opts_size].option; opts_size++)
+    ;
+
+  merged_options_size = def_opts_size + opts_size;
+  merged_options = (XrmOptionDescRec *)
+    malloc ((merged_options_size + 1) * sizeof(*default_options));
+  memcpy (merged_options, default_options,
+	  (def_opts_size * sizeof(*default_options)));
+  memcpy (merged_options + def_opts_size, options,
+	  ((opts_size + 1) * sizeof(*default_options)));
+
+  for (def_defaults_size = 0; default_defaults[def_defaults_size];
+       def_defaults_size++)
+    ;
+  for (defaults_size = 0; defaults[defaults_size]; defaults_size++)
+    ;
   merged_defaults = (char **)
-    malloc (sizeof (default_defaults) + (defaults_size * sizeof (char *)));
-  memcpy (merged_defaults, default_defaults, sizeof (default_defaults));
-  memcpy ((merged_defaults - 1 +
-	   (sizeof (default_defaults) / sizeof (default_defaults[0]))),
-	  defaults, ((defaults_size + 1) * sizeof (defaults[0])));
+    malloc ((def_defaults_size + defaults_size + 1) * sizeof (*defaults));;
+  memcpy (merged_defaults, default_defaults,
+	  def_defaults_size * sizeof(*defaults));
+  memcpy (merged_defaults + def_defaults_size, defaults,
+	  (defaults_size + 1) * sizeof(*defaults));
 }
 
 
@@ -99,9 +116,7 @@ merge_options P((void))
  */
 
 static int
-screenhack_ehandler (dpy, error)
-     Display *dpy;
-     XErrorEvent *error;
+screenhack_ehandler (Display *dpy, XErrorEvent *error)
 {
   fprintf (stderr, "\nX error in %s:\n", progname);
   if (XmuPrintDefaultErrorMessage (dpy, error, stderr))
@@ -112,20 +127,25 @@ screenhack_ehandler (dpy, error)
 }
 
 static Bool
-MapNotify_event_p (dpy, event, window)
-     Display *dpy;
-     XEvent *event;
-     XPointer window;
+MapNotify_event_p (Display *dpy, XEvent *event, XPointer window)
 {
   return (event->xany.type == MapNotify &&
 	  event->xvisibility.window == (Window) window);
 }
 
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+#ifdef USE_GL
+extern Visual *get_gl_visual (Screen *, const char *, const char *);
+#endif
+
+#ifdef XLOCKMORE
+extern void pre_merge_options (void);
+#endif
+
+
+
+int
+main (int argc, char **argv)
 {
   XtAppContext app;
   Widget toplevel;
@@ -135,7 +155,12 @@ main (argc, argv)
   Colormap cmap;
   Bool root_p;
   XEvent event;
+  Boolean dont_clear /*, dont_map */;
+  char version[255];
 
+#ifdef XLOCKMORE
+  pre_merge_options ();
+#endif
   merge_options ();
   toplevel = XtAppInitialize (&app, progclass, merged_options,
 			      merged_options_size, &argc, argv,
@@ -144,12 +169,29 @@ main (argc, argv)
   db = XtDatabase (dpy);
   XtGetApplicationNameAndClass (dpy, &progname, &progclass);
   XSetErrorHandler (screenhack_ehandler);
+
+  {
+    char *v = (char *) strdup(strchr(screensaver_id, ' '));
+    char *s = (char *) strchr(v, ',');
+    *s = 0;
+    sprintf (version, "%s: from the XScreenSaver%s distribution.",
+	     progclass, v);
+    free(v);
+  }
+
   if (argc > 1)
     {
+      const char *s;
       int i;
       int x = 18;
       int end = 78;
-      fprintf (stderr, "%s: unrecognised option \"%s\"\n", progname, argv[1]);
+      Bool help_p = !strcmp(argv[1], "-help");
+      fprintf (stderr, "%s\n", version);
+      for (s = progclass; *s; s++) fprintf(stderr, " ");
+      fprintf (stderr, "  http://people.netscape.com/jwz/xscreensaver/\n\n");
+
+      if (!help_p)
+	fprintf(stderr, "Unrecognised option: %s\n", argv[1]);
       fprintf (stderr, "Options include: ");
       for (i = 0; i < merged_options_size; i++)
 	{
@@ -167,9 +209,11 @@ main (argc, argv)
 	  if (i != merged_options_size - 1) fprintf (stderr, ", ");
 	}
       fprintf (stderr, ".\n");
-      exit (1);
+      exit (help_p ? 0 : 1);
     }
 
+  dont_clear = get_boolean_resource ("dontClearRoot", "Boolean");
+/*dont_map = get_boolean_resource ("dontMapWindow", "Boolean"); */
   mono_p = get_boolean_resource ("mono", "Boolean");
   if (CellsOfScreen (DefaultScreenOfDisplay (dpy)) <= 2)
     mono_p = True;
@@ -186,50 +230,83 @@ main (argc, argv)
     }
   else
     {
-      visual = get_visual_resource (dpy, "visualID", "VisualID");
+      Boolean def_visual_p;
+      Screen *screen = XtScreen (toplevel);
 
-      XtVaSetValues (toplevel, XtNmappedWhenManaged, False, 0);
-      XtRealizeWidget (toplevel);
-      window = XtWindow (toplevel);
+#ifdef USE_GL
+      visual = get_gl_visual (screen, "visualID", "VisualID");
+#else
+      visual = get_visual_resource (screen, "visualID", "VisualID", False);
+#endif
 
-      if (visual != DefaultVisualOfScreen (XtScreen (toplevel)))
+      if (toplevel->core.width <= 0)
+	toplevel->core.width = 600;
+      if (toplevel->core.height <= 0)
+	toplevel->core.height = 480;
+
+      def_visual_p = (visual == DefaultVisualOfScreen (screen));
+
+      if (!def_visual_p)
 	{
-	  Arg av [20];
-	  int ac;
 	  unsigned int bg, bd;
 	  Widget new;
-	  cmap = XCreateColormap (dpy, window, visual, AllocNone);
+
+	  cmap = XCreateColormap (dpy, RootWindowOfScreen(screen),
+				  visual, AllocNone);
 	  bg = get_pixel_resource ("background", "Background", dpy, cmap);
 	  bd = get_pixel_resource ("borderColor", "Foreground", dpy, cmap);
-	  ac = 0;
-	  XtSetArg (av[ac], XtNvisual, visual); ac++;
-	  XtSetArg (av[ac], XtNcolormap, cmap); ac++;
-	  XtSetArg (av[ac], XtNdepth, get_visual_depth (dpy, visual)); ac++;
-	  XtSetArg (av[ac], XtNbackground, (Pixel) bg); ac++;
-	  XtSetArg (av[ac], XtNborderColor, (Pixel) bd); ac++;
-	  new = XtAppCreateShell (progname, progclass,
-				  topLevelShellWidgetClass, dpy,
-				  av, ac);
+
+	  new = XtVaAppCreateShell (progname, progclass,
+				    topLevelShellWidgetClass, dpy,
+				    XtNmappedWhenManaged, False,
+				    XtNvisual, visual,
+				    XtNdepth, visual_depth (screen, visual),
+				    XtNwidth, toplevel->core.width,
+				    XtNheight, toplevel->core.height,
+				    XtNcolormap, cmap,
+				    XtNbackground, (Pixel) bg,
+				    XtNborderColor, (Pixel) bd,
+				    0);
 	  XtDestroyWidget (toplevel);
 	  toplevel = new;
-	}
-      else if (get_boolean_resource ("installColormap", "InstallColormap"))
-	{
-	  cmap = XCreateColormap (dpy, window,
-				  DefaultVisualOfScreen (XtScreen (toplevel)),
-				  AllocNone);
-	  XSetWindowColormap (dpy, window, cmap);
+	  XtRealizeWidget (toplevel);
+	  window = XtWindow (toplevel);
 	}
       else
 	{
-	  cmap = DefaultColormap (dpy, DefaultScreen (dpy));
+	  XtVaSetValues (toplevel, XtNmappedWhenManaged, False, 0);
+	  XtRealizeWidget (toplevel);
+	  window = XtWindow (toplevel);
+
+	  if (get_boolean_resource ("installColormap", "InstallColormap"))
+	    {
+	      cmap = XCreateColormap (dpy, window,
+				   DefaultVisualOfScreen (XtScreen (toplevel)),
+				      AllocNone);
+	      XSetWindowColormap (dpy, window, cmap);
+	    }
+	  else
+	    {
+	      cmap = DefaultColormap (dpy, DefaultScreen (dpy));
+	    }
 	}
 
-      XtPopup (toplevel, XtGrabNone);
-      window = XtWindow (toplevel);
+/*
+      if (dont_map)
+	{
+	  XtVaSetValues (toplevel, XtNmappedWhenManaged, False, 0);
+	  XtRealizeWidget (toplevel);
+	}
+      else
+*/
+	{
+	  XtPopup (toplevel, XtGrabNone);
+	}
+
+      XtVaSetValues(toplevel, XtNtitle, version, 0);
     }
 
-  if (!get_boolean_resource ("dontClearWindow", "Boolean")) /* kludge-o-rama */
+  if (!dont_clear)
     {
       XSetWindowBackground (dpy, window,
 			    get_pixel_resource ("background", "Background",
@@ -237,11 +314,12 @@ main (argc, argv)
       XClearWindow (dpy, window);
     }
 
-  if (!root_p && toplevel->core.mapped_when_managed)
+  if (!root_p)
     /* wait for it to be mapped */
     XIfEvent (dpy, &event, MapNotify_event_p, (XPointer) window);
 
   XSync (dpy, False);
   srandom ((int) time ((time_t *) 0));
-  screenhack (dpy, window);
+  screenhack (dpy, window); /* doesn't return */
+  return 0;
 }
